#include <SD.h>
#include <SPI.h>
#include <Servo.h>


int counter = 0;
unsigned int t = 100;
unsigned long period = 500;
int sped_up = 0;

//For ServoMotor
const int servoPin = 3;  // Pin connected to the servo
Servo myServo;           // Servo object

// For Melody and sounds
// Change this to make the song slower or faster
int tempo = 85;

// Select pin you want to use 
int buzzer = 2; // Digital signal

// This calculates the period of a whole note in ms
int wholenote = (60000 * 4) / tempo;

int divider = 0, noteDuration = 0, frequency = 0;


// INIT CARD MICROSD

File myFile;

// Change this to match the SD shield or module;
const int chipSelect = 10;

//How many notes I will read from the file
int maxNotes = 0;

void setup(){
  // put your setup code here, to run once:

  Serial.begin(9600);
  while (!Serial){
    ; // wait for serial port to connect. 
  }

  // Init buzzer pin
  pinMode(2, OUTPUT);


  Serial.print("Initializing SD card...");

  if (!SD.begin(chipSelect)){
    Serial.println("initialization failed!");
    return;
  }

  Serial.println("initialization done.");

  // Open the file for reading:
  myFile = SD.open("note.txt", FILE_READ);

    if (myFile){
    int noteCount = 0;
    maxNotes = myFile.readStringUntil('\n').toInt();
    // The Matrix where we put the values for frequency and divider
    int noteData[maxNotes][2] ={0}; 

    // Read the note data from the file and store it in the matrix
    while (myFile.available() && noteCount < maxNotes){
      frequency = myFile.readStringUntil('\n').toInt();
      divider = myFile.readStringUntil('\n').toInt();

      noteData[noteCount][0] = frequency;
      noteData[noteCount][1] = divider;

      noteCount++;
    }

    // close the file:
    myFile.close();
    
    for (int i = 0; i < noteCount; i++){

      frequency = noteData[i][0];
      divider = noteData[i][1];

      // Compute the duration of each note
      if (divider > 0){
        // regular note, just proceed
        noteDuration = (wholenote) / divider;
      }
      else if (divider < 0){
        // dotted notes are represented with negative durations
        noteDuration = (wholenote) / abs(divider);
        noteDuration *= 1.5; // increases the duration in half for dotted notes
      }

      // Play the note
      tone(buzzer, frequency, noteDuration * 0.9); // Give the digital signal through the buzzer/ to the tesla coil

      // Wait for the specified duration before playing the next note.
      delay(noteDuration);

      // Stop the waveform generation before the next note.
      noTone(buzzer);
    }

  }else{
    // if the file didn't open, print an error:
    Serial.println("error opening note.txt for reading");
  }

  //For ServoMotor 
     myServo.attach(servoPin);
}

void loop(){

  // Choose your dance style 
  danceServo(2);
  // put your main code here, to run repeatedly:
}

void danceServo(int pattern) {
  switch (pattern) {
    case 1:  // Example dance pattern 1
      // Move the servo to position 0 degrees
      myServo.write(0);
      delay(500);  // Pause for half a second
      // Move the servo to position 180 degrees
      myServo.write(180);
      delay(500);  // Pause for half a second
      break;
    case 2:  // Example dance pattern 2
      // Move the servo to position 90 degrees
      myServo.write(90);
      delay(1000);  // Pause for half a second
      // Move the servo to position 0 degrees
      myServo.write(0);
      delay(1000);  // Pause for half a second
      // Move the servo to position 180 degrees
      myServo.write(180);
      delay(1000);  // Pause for half a second
      break;
    default:  // Default dance pattern
      // Move the servo to position 0 degrees
      myServo.write(0);
      delay(200);  // Pause for two-tenths of a second
      // Move the servo to position 180 degrees
      myServo.write(180);
      delay(200);  // Pause for two-tenths of a second
      break;
  }
}